-- @release $Id: Priority.lua 447 2016-08-31 11:31:59Z reighnman $

if not SlamAndAwe then return end

local L = LibStub("AceLocale-3.0"):GetLocale("SlamAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
if not C then
  SlamAndAwe:SetConstants()
end
local LBF = LibStub("LibButtonFacade", true)
local LBFgroup = nil
local _, _, _, clientVersion = GetBuildInfo()
local minaoetargets = 3
--if LBF then
-- local LBFgroup = LBF:Group("SlamAndAwe")
--end
SlamAndAwe.PriorityFrame = CreateFrame("Button", "SAA_PriorityFrame", UIParent)
SlamAndAwe.PriorityFrame.cooldown = CreateFrame("Cooldown", "SAA_PriorityFrameCooldown", SlamAndAwe.PriorityFrame, "CooldownFrameTemplate") --Inherit from WoW template (texture not necessary)

---------------------------
-- Local Variables
---------------------------

SlamAndAwe.priorityTable = {}
SlamAndAwe.priorityTable.name = {}
SlamAndAwe.priorityTable.icon = {}
SlamAndAwe.priorityTable.test = {}

-- table.foreach(SlamAndAwe.priorityTable.name, print)

--------------------------
-- Helper Functions
--------------------------
-- returns : true if spellname is available in less than priority.cooldown /run print(SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Mortal Strike ID"]))
-- or if spellname is currently activated
-- , remaining cooldown, 0 if ready to be cast
function SlamAndAwe:SpellAvailable(spellid)
  if SlamAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
    return false
  end
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()
    return timeleft <= SlamAndAwe.db.char.priority.cooldown, duration, timeleft
  end
  return false, 999
end

function SlamAndAwe:SpellReady(spellid)
  if SlamAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
    return false
  end
  local available = GetSpellInfo(GetSpellInfo(spellid))
  local usable = IsUsableSpell(spellid)
  if available and usable then
    return true, usable
  end
  return false, false
end

function SlamAndAwe:SpellCooldown(spellid) --/run print(GetSpellCooldown(SlamAndAwe.constants["Bladestorm ID"]))
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()
    return duration, timeleft
  end
  return false, 999
end

function SlamAndAwe:PrintPriorities()
  for index = 1, 30 do
    SlamAndAwe:Print(SlamAndAwe.db.char.priority.prOption[index])
  end
end

function SlamAndAwe:IsBoss()
  local mylevel = UnitLevel("player")
  local bosslevel = mylevel + 3
  if UnitLevel("target") >= bosslevel or UnitClassification("target") == "worldboss" then
    return true
  end
end

function SlamAndAwe:GCDAvailable()
  local startTime, gcdDuration = GetSpellCooldown(SlamAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  return gcdTime <= SlamAndAwe.db.char.priority.cooldown
end

function SlamAndAwe:IsOffGCD()
  local startTime, gcdDuration = GetSpellCooldown(SlamAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  if gcdTime > 0 then
    return true
  else
    return false
  end
end

function SlamAndAwe:GetDebuffInfo(debuff)
  local index = 1
  while UnitDebuff("target", index) do
    local name, _, _, count, _, _, debuffExpires, unitCaster, _, _, id = UnitDebuff("target", index)
    local isMine = unitCaster == "player"
    if id == debuff and isMine then
      local duration = debuffExpires - GetTime()
      return duration > SlamAndAwe.db.char.priority.cooldown, duration, count, debuffExpires
    end
    index = index + 1
  end
  return false, 0, 0, 0
end

function SlamAndAwe:GetBuffInfo(buff)
  local index = 1
  while UnitBuff("player", index) do
    local name, _, _, count, _, _, buffExpires, _, _, _, id = UnitBuff("player", index)
    if id == buff then
      local duration = buffExpires - GetTime()
      return duration > SlamAndAwe.db.char.priority.cooldown, duration, count
    end
    index = index + 1
  end
  return false, 0, 0
end

function SlamAndAwe:IsTargetImmune(spell)
  local target = UnitName("target") or ""
  if spell and SlamAndAwe.db.char.immuneTargets[target.."_"..spell] then
    return true
  end
  return false
end

local minRange = 0
local items = {
  {5, 37727}, -- Ruby Acorn
  {6, 63427}, -- Worgsaw
  {8, 34368}, -- Attuned Crystal Cores /run print(IsItemInRange("Attuned Crystal Cores", "TARGET"))
  {10, 32321}, -- Sparrowhawk Net
  {15, 33069}, -- Sturdy Rope
  {20, 10645}, -- Gnomish Death Ray
  {25, 24268}, -- Netherweave Net
  {30, 835}, -- Large Rope Net
  {35, 24269}, -- Heavy Netherweave Net
  {40, 28767}, -- The Decapitator
  {45, 23836}, -- Goblin Rocket Launcher
  {50, 116139}, -- Haunting Memento
  {60, 32825}, -- Soul Cannon
  {70, 41265}, -- Eyesore Blaster
  {80, 35278}, -- Reinforced Net
  {100, 33119}, -- Malister's Frost Wand
}

function SlamAndAwe:CountNameplates(maxRange)
  local last = result or 0
  local counter = 0

  for i = 1, 40 do
    local unit = "nameplate"..i
    if UnitExists(unit) and not UnitIsFriend("player", unit) then
      local min, max = SlamAndAwe:GetRange(unit)
      if min and max and min >= minRange and max <= maxRange then
        counter = counter + 1
      end
    end
  end
  if counter > 0 then
    return counter
  else
    return 0
  end
end

function SlamAndAwe:GetRange(unit)
  if not UnitExists(unit) or UnitIsFriend("player", unit) then
    return nil
  end

  local minRange, maxRange = 0, 0
  for _, items in ipairs(items) do
    local range, item = items[1], items[2]
    if IsItemInRange("item:"..item, unit) then
      maxRange = range
      break
    end
    minRange = range
  end
  return minRange, maxRange
end

------------------
-- Misc Functions
------------------

function SlamAndAwe:EstimateIgnorePain()
  local versatilityDamageBonus = GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE);
  local versatilityDamangeBonusPerc = (1 + versatilityDamageBonus * 10 ^ -2)
  local curRage = UnitPower("player", "rage")

  if curRage >= 20 then
    local _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, currentIP = UnitBuff("Player","Ignore Pain")
    if not currentIP then
      currentIP = 0
    end
    local _, STR, _, _ = UnitStat("player", 1)
    local APbase, APposBuff, APnegBuff = UnitAttackPower("player")
    local APfinal = APbase + APposBuff + APnegBuff
    local ipMaxRage = 60
    return floor((7/15*min(curRage,ipMaxRage)*APfinal)*versatilityDamangeBonusPerc*SlamAndAwe.db.char.ipMultiplier*.9+currentIP)
  else
    return 0
  end
end

------------------
-- Priorities
------------------

function SlamAndAwe:SetPriorityTable()
  SlamAndAwe.priorityTable.name["none"] = L["None"]
  _, _, SlamAndAwe.priorityTable.icon["none"] = "Interface/Tooltips/UI-Tooltip-Background"
  SlamAndAwe.priorityTable.test["none"] = function () return false end
end

--SlamAndAwe

SlamAndAwe.priorityTable.name["cs"] = SlamAndAwe.constants["Colossus Smash"]
SlamAndAwe.priorityTable.icon["cs"] = SlamAndAwe.constants["Colossus Smash Icon"]
SlamAndAwe.priorityTable.test["cs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Colossus Smash ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
  end
end

SlamAndAwe.priorityTable.name["cs_ncs"] = "Colossus Smash (if=!debuff.colossus_smash)"
SlamAndAwe.priorityTable.icon["cs_ncs"] = SlamAndAwe.constants["Colossus Smash Icon"]
SlamAndAwe.priorityTable.test["cs_ncs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Colossus Smash ID"]) then
    local CS, CSduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if not CS then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["cs_apl1"] = "Colossus Smash (if=!buff.precise_strikes & !buff.shattered_defenses)"
SlamAndAwe.priorityTable.icon["cs_apl1"] = SlamAndAwe.constants["Colossus Smash Icon"]
SlamAndAwe.priorityTable.test["cs_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Colossus Smash ID"]) then
    local PS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Precise Strikes ID"])
    local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
    if not PS and not SD then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["cs_re"] = "Colossus Smash (if=debuff.colossus_smash.remains <= 2)"
SlamAndAwe.priorityTable.icon["cs_re"] = SlamAndAwe.constants["Colossus Smash Icon"]
SlamAndAwe.priorityTable.test["cs_re"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Colossus Smash ID"]) then
    local CS, CSduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if (not CS or CSduration <= 2) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend"] = "Rend (if=!debuff.rend)"
SlamAndAwe.priorityTable.icon["rend"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rend ID"]) then
    local REND, RENDduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    if not REND then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend_03"] = "Rend (if=debuff.rend <= duration * 0.3)"
SlamAndAwe.priorityTable.icon["rend_03"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend_03"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rend ID"]) then
    local RE, REremaining = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    local calc = 15 * 0.3
    if REremaining <= calc then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend_2s"] = "Rend (if=!debuff.rend or duration <= 2)"
SlamAndAwe.priorityTable.icon["rend_2s"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend_2s"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rend ID"]) then
    local REND, RENDduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    if (not REND or RENDduration <= 2) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend_2s20"] = "Rend (if=subtwenty & !debuff.rend or duration <= 2)"
SlamAndAwe.priorityTable.icon["rend_2s20"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend_2s20"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rend ID"]) and SlamAndAwe:TargetSubTwenty() then
    local REND, RENDduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    if (not REND or RENDduration <= 2) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend_2ns20"] = "Rend (if=!subtwenty & !debuff.rend or duration <= 2)"
SlamAndAwe.priorityTable.icon["rend_2ns20"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend_2ns20"] =
function ()
  if SlamAndAwe:SpellReady(spellid) and SlamAndAwe:TargetSubTwenty() then
    local REND, RENDduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    if (not REND or RENDduration <= 2) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rend_5s"] = "Rend (if=!debuff.rend or duration <= 5)"
SlamAndAwe.priorityTable.icon["rend_5s"] = SlamAndAwe.constants["Rend Icon"]
SlamAndAwe.priorityTable.test["rend_5s"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rend ID"]) then
    local REND, RENDduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Rend ID"])
    if (not REND or RENDduration <= 5) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rend ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["cle"] = SlamAndAwe.constants["Cleave"]
SlamAndAwe.priorityTable.icon["cle"] = SlamAndAwe.constants["Cleave Icon"]
SlamAndAwe.priorityTable.test["cle"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Cleave ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Cleave ID"])
  end
end

SlamAndAwe.priorityTable.name["bt"] = SlamAndAwe.constants["Bloodthirst"]
SlamAndAwe.priorityTable.icon["bt"] = SlamAndAwe.constants["Bloodthirst Icon"]
SlamAndAwe.priorityTable.test["bt"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodthirst ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodthirst ID"])
  end
end

SlamAndAwe.priorityTable.name["bt_ir"] = "Bloodthirst (if=!inner_rage)"
SlamAndAwe.priorityTable.icon["bt_ir"] = SlamAndAwe.constants["Bloodthirst Icon"]
SlamAndAwe.priorityTable.test["bt_ir"] =
function ()
  if not SlamAndAwe.db.char.talInnerRage then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodthirst ID"]) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodthirst ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bt_en"] = "Bloodthirst (if=!buff.enraged)"
SlamAndAwe.priorityTable.icon["bt_en"] = SlamAndAwe.constants["Bloodthirst Icon"]
SlamAndAwe.priorityTable.test["bt_en"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodthirst ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if not Enraged then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodthirst ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bt_tfb"] = "Bloodthirst (if=taste_for_blood.stack > 3)"
SlamAndAwe.priorityTable.icon["bt_tfb"] = SlamAndAwe.constants["Bloodthirst Icon"]
SlamAndAwe.priorityTable.test["bt_tfb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodthirst ID"]) then
    local TFB, TFBduration, TFBcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Taste for Blood ID"])
    if TFBcount > 3 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodthirst ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bt_tfb6"] = "Bloodthirst (if=taste_for_blood.stack = 6)"
SlamAndAwe.priorityTable.icon["bt_tfb6"] = SlamAndAwe.constants["Bloodthirst Icon"]
SlamAndAwe.priorityTable.test["bt_tfb6"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodthirst ID"]) then
    local TFB, TFBduration, TFBcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Taste for Blood ID"])
    if TFBcount == 6 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodthirst ID"])
    end
  end
end

--about to do this

SlamAndAwe.priorityTable.name["fr"] = SlamAndAwe.constants["Focused Rage"]
SlamAndAwe.priorityTable.icon["fr"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
    local rage = UnitPower("player", "rage")
    if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["fr_ven"] = "Focused Rage (if=buff.vengeance)"
SlamAndAwe.priorityTable.icon["fr_ven"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr_ven"] =
function ()
  if SlamAndAwe.db.char.talVengeance then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
      local venge = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Vengeance: Focused Rage ID"])
      if venge then
        if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
          return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
        end
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fr_apl1"] = "Focused Rage (if=buff.fr < 3 | (talDeadyCalm & buff.bc))"
SlamAndAwe.priorityTable.icon["fr_apl1"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
    local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
    local BC, BCduration, BCcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    if (FRcount < 3 or (talDeadyCalm and BC)) then
      if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
      end
    end
  else
    if talDeadyCalm and BC then
      if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fr_90"] = "Focused Rage (if=rage >= 90 | buff.ultimatum)"
SlamAndAwe.priorityTable.icon["fr_90"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr_90"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
    local rage = UnitPower("player", "rage")
    local Ultimatum = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Ultimatum ID"])
    if (rage > 90 or Ultimatum) then
      if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fr_28"] = "Focused Rage (if=tal.dauntless & rage >= 28 )"
SlamAndAwe.priorityTable.icon["fr_28"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr_28"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
    local rage = UnitPower("player", "rage")
    if (rage >= 28 and SlamAndAwe.db.char.talDauntless) then
      if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fr_35"] = "Focused Rage (if=rage >= 35 )"
SlamAndAwe.priorityTable.icon["fr_35"] = SlamAndAwe.constants["Focused Rage Icon"]
SlamAndAwe.priorityTable.test["fr_35"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Focused Rage ID"]) then
    local rage = UnitPower("player", "rage")
    if rage >= 35 then
      if SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Focused Rage ID"]) == 0 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Focused Rage ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["rb"] = SlamAndAwe.constants["Raging Blow"]
SlamAndAwe.priorityTable.icon["rb"] = SlamAndAwe.constants["Raging Blow Icon"]
SlamAndAwe.priorityTable.test["rb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Raging Blow ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Raging Blow ID"])
  end
end

SlamAndAwe.priorityTable.name["rb_ir"] = "Raging Blow (if=tal.inner_rage)"
SlamAndAwe.priorityTable.icon["rb_ir"] = SlamAndAwe.constants["Raging Blow Icon"]
SlamAndAwe.priorityTable.test["rb_ir"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Raging Blow ID"]) and SlamAndAwe.db.char.talInnerRage then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Raging Blow ID"])
  end
end

SlamAndAwe.priorityTable.name["rb_en"] = "Raging Blow (if=buff.enraged)"
SlamAndAwe.priorityTable.icon["rb_en"] = SlamAndAwe.constants["Raging Blow Icon"]
SlamAndAwe.priorityTable.test["rb_en"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Raging Blow ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    return Enraged and SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Raging Blow ID"])
  end
end

SlamAndAwe.priorityTable.name["sl"] = SlamAndAwe.constants["Slam"]
SlamAndAwe.priorityTable.icon["sl"] = SlamAndAwe.constants["Slam Icon"]
SlamAndAwe.priorityTable.test["sl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Slam ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Slam ID"])
  end
end

SlamAndAwe.priorityTable.name["sl_cs"] = "Slam (if=rage>=70 | debuff.colossus_smash.up) & health.pct>=20)"
SlamAndAwe.priorityTable.icon["sl_cs"] = SlamAndAwe.constants["Slam Icon"]
SlamAndAwe.priorityTable.test["sl_cs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Slam ID"]) and not SlamAndAwe:TargetSubTwenty() then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local rage = UnitPower("player", "rage")
    if (rage >= 70 and CS) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Slam ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["sl_apl1"] = "Slam (if=!tal.fob & (debuff.cs | rage.deficit < 40) & !tal.fr | tal.dc & buff.bc)"
SlamAndAwe.priorityTable.icon["sl_apl1"] = SlamAndAwe.constants["Slam Icon"]
SlamAndAwe.priorityTable.test["sl_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Slam ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local CL, CLduration, CLcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Cleave ID"])
    local BC, BCduration, BCcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local rage = UnitPower("player", "rage")
    local maxrage = UnitPowerMax("player" , "rage")
    if ((CS or rage > maxrage - 40) and not talFervorOfBattle and not talFocusedRage) or (talDeadlyCalm and BC) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Slam ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["sl_apl2"] = "Slam (if=!tal.fob & (!tal.fr | rage>=100 | fr.stack=3))"
SlamAndAwe.priorityTable.icon["sl_apl2"] = SlamAndAwe.constants["Slam Icon"]
SlamAndAwe.priorityTable.test["sl_apl2"] =
function ()
  if not talFervorOfBattle then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Slam ID"]) then
      local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
      local rage = UnitPower("player", "rage")
      if rage >= 100 or FRcount == 3 or not talFocusedRage then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Slam ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fur"] = SlamAndAwe.constants["Furious Slash"]
SlamAndAwe.priorityTable.icon["fur"] = SlamAndAwe.constants["Furious Slash Icon"]
SlamAndAwe.priorityTable.test["fur"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Furious Slash ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Furious Slash ID"])
  end
end

SlamAndAwe.priorityTable.name["fur_fr"] = ("Furious Slash (if=buff.frenzy.duration <= 2)")
SlamAndAwe.priorityTable.icon["fur_fr"] = SlamAndAwe.constants["Furious Slash Icon"]
SlamAndAwe.priorityTable.test["fur_fr"] =
function ()
  if SlamAndAwe.db.char.talFrenzy then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Furious Slash ID"]) then
      local FR, FRduration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Frenzy ID"])
      if FRduration <= 2 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Furious Slash ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fur_fr3"] = ("Furious Slash (if=buff.frenzy.duration <= 2 | stack < 3)")
SlamAndAwe.priorityTable.icon["fur_fr3"] = SlamAndAwe.constants["Furious Slash Icon"]
SlamAndAwe.priorityTable.test["fur_fr3"] =
function ()
  if SlamAndAwe.db.char.talFrenzy then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Furious Slash ID"]) then
      local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Frenzy ID"])
      if (FRduration <= 2 or FRcount < 3) then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Furious Slash ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["fur_tfb"] = "Furious Slash (if=tfb.stack < 3)"
SlamAndAwe.priorityTable.icon["fur_tfb"] = SlamAndAwe.constants["Furious Slash Icon"]
SlamAndAwe.priorityTable.test["fur_tfb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Furious Slash ID"]) then
    local TFB, TFBduration, TFBcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Taste for Blood ID"])
    if TFBcount < 3 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Furious Slash ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["br"] = "Berserker Rage (if=!buff.enrage & off.gcd)"
SlamAndAwe.priorityTable.icon["br"] = SlamAndAwe.constants["Berserker Rage Icon"]
SlamAndAwe.priorityTable.test["br"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Berserker Rage ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if not Enraged and SlamAndAwe:IsOffGCD() then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Berserker Rage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["br_out"] = "Berserker Rage (if=talent.outburst & cooldown.dragon_roar=0 & !buff.enrage)"
SlamAndAwe.priorityTable.icon["br_out"] = SlamAndAwe.constants["Berserker Rage Icon"]
SlamAndAwe.priorityTable.test["br_out"] =
function ()
  if SlamAndAwe.db.char.talOutburst then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Berserker Rage ID"]) then
      local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
      local dr_dur, dr_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Dragon Roar ID"])
      if dr_dur == 0 and not Enraged then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Berserker Rage ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["bc"] = "Battle Cry (if=off.gcd)"
SlamAndAwe.priorityTable.icon["bc"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
  end
end

SlamAndAwe.priorityTable.name["bc_dr"] = "Battle Cry (if=buff.dragon_roar & off.gcd)"
SlamAndAwe.priorityTable.icon["bc_dr"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc_dr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    local DR = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Dragon Roar ID"])
    if DR then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bc_en"] = "Battle Cry (if=buff.enrage & off.gcd))"
SlamAndAwe.priorityTable.icon["bc_en"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc_en"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if Enraged then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bc_apl"] = "Battle Cry (if=odyns_fury.cd=0 & (bloodthirst.cd=0 | (enrage.dur>bloodthirst.cd)) | !odyns_fury)"
SlamAndAwe.priorityTable.icon["bc_apl"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc_apl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    local odf_available = GetSpellInfo(GetSpellInfo(SlamAndAwe.constants["Odyn's Fury ID"]))
    local odf_dur, odf_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Odyn's Fury ID"])
    local odf_dur, odf_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Bloodthirst ID"])
    local en, en_dur = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if ((odf_available and odf_dur == 0 and (bt_dur == 0 or (en_dur > bt_cd))) or not odf_available ) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bc_apl2"] = "Battle Cry (if=debuff.cs >=5 | (debuff.cs & cs.avail))"
SlamAndAwe.priorityTable.icon["bc_apl2"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc_apl2"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    local CS, CS_duration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local CSM_dur, CSM_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Colossus Smash ID"])
    local CS_avail = SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
    if (CS_duration >= 5 or (CS and CS_avail)) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["bc_cs"] = "Battle Cry (if=(debuff.cs>=5 | debuff.cs & cs.cooldown=0) & off.gcd)"
SlamAndAwe.priorityTable.icon["bc_cs"] = SlamAndAwe.constants["Battle Cry Icon"]
SlamAndAwe.priorityTable.test["bc_cs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Battle Cry ID"]) and SlamAndAwe:IsOffGCD() then
    local cs_time, cs_dur, cs_en = GetSpellCooldown(GetSpellInfo(SlamAndAwe.constants["Colossus Smash ID"]))
    local cs_cd = (cs_time + cs_dur - GetTime())
    local CS, cs_duration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if cs_duration >= 5 or (CS and cs_cd <= 1.5) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["cosh"] = SlamAndAwe.constants["Commanding Shout (if self.health.pct<20)"]
SlamAndAwe.priorityTable.icon["cosh"] = SlamAndAwe.constants["Commanding Shout Icon"]
SlamAndAwe.priorityTable.test["cosh"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Commanding Shout ID"]) and SlamAndAwe:SelfSubTwenty() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Commanding Shout ID"])
  end
end

SlamAndAwe.priorityTable.name["ava"] = SlamAndAwe.constants["Avatar"]
SlamAndAwe.priorityTable.icon["ava"] = SlamAndAwe.constants["Avatar Icon"]
SlamAndAwe.priorityTable.test["ava"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Avatar ID"]) and SlamAndAwe:IsOffGCD() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Avatar ID"])
  end
end

SlamAndAwe.priorityTable.name["ava_dr"] = "Avatar on Boss (if=boss & buff.dragon_roar)"
SlamAndAwe.priorityTable.icon["ava_dr"] = SlamAndAwe.constants["Avatar Icon"]
SlamAndAwe.priorityTable.test["ava_dr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Avatar ID"]) and SlamAndAwe:IsOffGCD() and SlamAndAwe:IsBoss() then
    local DR = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Dragon Roar ID"])
    if DR then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Avatar ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ava_bbc"] = "Avatar on Boss (if=buff.battle_cry & off.gcd)"
SlamAndAwe.priorityTable.icon["ava_bbc"] = SlamAndAwe.constants["Avatar Icon"]
SlamAndAwe.priorityTable.test["ava_bbc"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Avatar ID"]) and SlamAndAwe:IsOffGCD() and SlamAndAwe:IsBoss() then
    local BC, BC_duration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    if BC then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Avatar ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ava_apl1"] = "Avatar on Boss (if=off.gcd & (debuff.cs >= 5 | (debuff.cs and cs.avail)))"
SlamAndAwe.priorityTable.icon["ava_apl1"] = SlamAndAwe.constants["Avatar Icon"]
SlamAndAwe.priorityTable.test["ava_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Avatar ID"]) and SlamAndAwe:IsOffGCD() and SlamAndAwe:IsBoss() then
    local CS, CS_duration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local CSM_dur, CSM_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Colossus Smash ID"])
    local CS_avail = SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Colossus Smash ID"])
    if (CS_duration >= 5 or (CS and CS_avail)) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Avatar ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ava_boss"] = "Avatar on Boss (if=boss & off.gcd)"
SlamAndAwe.priorityTable.icon["ava_boss"] = SlamAndAwe.constants["Avatar Icon"]
SlamAndAwe.priorityTable.test["ava_boss"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Avatar ID"]) and SlamAndAwe:IsOffGCD() and SlamAndAwe:IsBoss() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Avatar ID"])
  end
end

SlamAndAwe.priorityTable.name["blb"] = "Bloodbath (if=off.gcd)"
SlamAndAwe.priorityTable.icon["blb"] = SlamAndAwe.constants["Bloodbath Icon"]
SlamAndAwe.priorityTable.test["blb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodbath ID"]) and SlamAndAwe:IsOffGCD() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodbath ID"])
  end
end

SlamAndAwe.priorityTable.name["blb_dr"] = "Bloodbath (if=buff.dragon_roar & off.gcd))"
SlamAndAwe.priorityTable.icon["blb_dr"] = SlamAndAwe.constants["Bloodbath Icon"]
SlamAndAwe.priorityTable.test["blb_dr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodbath ID"]) and SlamAndAwe:IsOffGCD() then
    local DR = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Dragon Roar ID"])
    if DR then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodbath ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["blb_apl"] = "Bloodbath (if=buff.dragon_roar|!tal.dragon_roar & (buff.battle_cry | battle_cry.cd>10)) & off.gcd)"
SlamAndAwe.priorityTable.icon["blb_apl"] = SlamAndAwe.constants["Bloodbath Icon"]
SlamAndAwe.priorityTable.test["blb_apl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bloodbath ID"]) and SlamAndAwe:IsOffGCD() then
    local bc, bc_dur, bc_cd = SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Battle Cry ID"])
    local bc_time, bc_dur, bc_en = GetSpellCooldown(GetSpellInfo(SlamAndAwe.constants["Battle Cry ID"]))
    local bc_cd = (bc_time + bc_dur - GetTime())
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local DR = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Dragon Roar ID"])
    if DR then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodbath ID"])
    elseif not talDragonRoar and (BC or bc_cd > 10) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bloodbath ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hel_cs"] = "Heroic Leap (if=debuff.colossus_smash.up & off.gcd)"
SlamAndAwe.priorityTable.icon["hel_cs"] = SlamAndAwe.constants["Heroic Leap Icon"]
SlamAndAwe.priorityTable.test["hel_cs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Heroic Leap ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if CS and SlamAndAwe:IsOffGCD() then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Heroic Leap ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hel_sd"] = "Heroic Leap (if=!buff.shattered_dfenses & off.gcd)"
SlamAndAwe.priorityTable.icon["hel_sd"] = SlamAndAwe.constants["Heroic Leap Icon"]
SlamAndAwe.priorityTable.test["hel_sd"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Heroic Leap ID"]) then
    local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
    if not SD and SlamAndAwe:IsOffGCD() then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Heroic Leap ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hel"] = SlamAndAwe.constants["Heroic Leap"]
SlamAndAwe.priorityTable.icon["hel"] = SlamAndAwe.constants["Heroic Leap Icon"]
SlamAndAwe.priorityTable.test["hel"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Heroic Leap ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Heroic Leap ID"])
  end
end

SlamAndAwe.priorityTable.name["odf"] = SlamAndAwe.constants["Odyn's Fury"]
SlamAndAwe.priorityTable.icon["odf"] = SlamAndAwe.constants["Odyn's Fury Icon"]
SlamAndAwe.priorityTable.test["odf"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Odyn's Fury ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Odyn's Fury ID"])
  end
end

SlamAndAwe.priorityTable.name["odf_apl"] = "Odyn's Fury (if=battle_cry.up)"
SlamAndAwe.priorityTable.icon["odf_apl"] = SlamAndAwe.constants["Odyn's Fury Icon"]
SlamAndAwe.priorityTable.test["odf_apl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Odyn's Fury ID"]) then
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    if BC then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Odyn's Fury ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram"] = SlamAndAwe.constants["Rampage"]
SlamAndAwe.priorityTable.icon["ram"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
  end
end

SlamAndAwe.priorityTable.name["ram_apl1"] = "Rampage (if=!buff.enrage | (rage >= 100 & !buff.juggernaut) | buff.massacre)"
SlamAndAwe.priorityTable.icon["ram_apl1"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local rage = UnitPower("player", "rage")
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    local Juggernaut = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Juggernaut ID"])
    if (not Enarged or (rage >= 100 and not Juggernaut) or MASS) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_aoe"] = "Rampage (if=buff.meat_cleaver)"
SlamAndAwe.priorityTable.icon["ram_aoe"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_aoe"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local MC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Meat Cleaver ID"])
    if MC then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_ne"] = "Rampage (if=!buff.enrage)"
SlamAndAwe.priorityTable.icon["ram_ne"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_ne"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if not Enraged then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_mass"] = "Rampage (if=buff.massacre)"
SlamAndAwe.priorityTable.icon["ram_mass"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_mass"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    if MASS then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_mass95"] = "Rampage (if=buff.massacre | rage>95)"
SlamAndAwe.priorityTable.icon["ram_mass95"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_mass95"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    local rage = UnitPower("player", "rage")
    if (MASS or rage > 95) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_apl2"] = "Rampage (if=target.heath>20 & (battle_cry.cd>3 | battle_cry | rage>90))"
SlamAndAwe.priorityTable.icon["ram_apl2"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_apl2"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) and SlamAndAwe:TargetSubTwenty() then
    local bc_dur, bc_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Battle Cry ID"])
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local rage = UnitPower("player", "rage")
    if (bc_cd > 3 or BC or rage >= 90) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_mass70en"] = "Rampage (if=buff.massacre | rage>=70 & !buff.rampage)"
SlamAndAwe.priorityTable.icon["ram_mass70en"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_mass70en"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    local rage = UnitPower("player", "rage")
    if (MASS or rage >= 70) and not Enraged then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_mass100en"] = "Rampage (if=buff.massacre | rage>=100 & !buff.rampage)"
SlamAndAwe.priorityTable.icon["ram_mass100en"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_mass100en"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    local rage = UnitPower("player", "rage")
    if (MASS or rage >= 100) and not Enraged then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ram_mass100"] = "Rampage (if=buff.massacre | rage >= 100)"
SlamAndAwe.priorityTable.icon["ram_mass100"] = SlamAndAwe.constants["Rampage Icon"]
SlamAndAwe.priorityTable.test["ram_mass100"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Rampage ID"]) then
    local MASS = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Massacre ID"])
    local rage = UnitPower("player", "rage")
    if MASS or (rage >= 100) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Rampage ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["rav"] = SlamAndAwe.constants["Ravager"]
SlamAndAwe.priorityTable.icon["rav"] = SlamAndAwe.constants["Ravager Icon ID"]
SlamAndAwe.priorityTable.test["rav"] =
function ()
  if SlamAndAwe.db.char.talRavager then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Ravager ID"]) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Ravager ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["stb"] = SlamAndAwe.constants["Storm Bolt"]
SlamAndAwe.priorityTable.icon["stb"] = SlamAndAwe.constants["Storm Bolt Icon"]
SlamAndAwe.priorityTable.test["stb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Storm Bolt ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Storm Bolt ID"])
  end
end

SlamAndAwe.priorityTable.name["drr"] = SlamAndAwe.constants["Dragon Roar"]
SlamAndAwe.priorityTable.icon["drr"] = SlamAndAwe.constants["Dragon Roar Icon"]
SlamAndAwe.priorityTable.test["drr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Dragon Roar ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Dragon Roar ID"])
  end
end

SlamAndAwe.priorityTable.name["drr_apl"] = "Dragon Roar (if=!Bloodbath.en & (battle_cry.cd<1|>10) | Bloodbath.en & Bloodbath.cd=0)"
SlamAndAwe.priorityTable.icon["drr_apl"] = SlamAndAwe.constants["Dragon Roar Icon"]
SlamAndAwe.priorityTable.test["drr_apl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Dragon Roar ID"]) then
    local bc_time, bc_dur, bc_en = GetSpellCooldown(GetSpellInfo(SlamAndAwe.constants["Battle Cry ID"]))
    local _, bb_dur, _ = GetSpellCooldown(GetSpellInfo(SlamAndAwe.constants["Bloodbath ID"]))
    local bc_cd = (bc_time + bc_dur - GetTime())
    if not SlamAndAwe.db.char.talBloodbath then
      if (bc_cd < 1 or bc_cd > 10) then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Dragon Roar ID"])
      end
    elseif (SlamAndAwe.db.char.talBloodbath and bb_cd == 0) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Dragon Roar ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hc"] = "Charge (if=heroic_leap.cd = 0)"
SlamAndAwe.priorityTable.icon["hc"] = SlamAndAwe.constants["Charge Icon"]
SlamAndAwe.priorityTable.test["hc"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Charge ID"]) then
    local hl_dur, hl_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Heroic Leap ID"])
    if hl_dur == 0 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Charge ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hc_s20"] = "Charge (if=heroic_leap.cd = 0)"
SlamAndAwe.priorityTable.icon["hc_s20"] = SlamAndAwe.constants["Charge Icon"]
SlamAndAwe.priorityTable.test["hc_s20"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Charge ID"]) and SlamAndAwe:TargetSubTwenty() then
    local hl_dur, hl_cd = SlamAndAwe:SpellCooldown(SlamAndAwe.constants["Heroic Leap ID"])
    if hl_dur == 0 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Charge ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["imv"] = "Impending Victory (if=target.health.pct>20)"
SlamAndAwe.priorityTable.icon["imv"] = SlamAndAwe.constants["Impending Vicotry Icon"]
SlamAndAwe.priorityTable.test["imv"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Impending Victory ID"]) and not SlamAndAwe:TargetSubTwenty() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Impending Vicotry ID"])
  end
end

SlamAndAwe.priorityTable.name["shw"] = SlamAndAwe.constants["Shockwave"]
SlamAndAwe.priorityTable.icon["shw"] = SlamAndAwe.constants["Shockwave Icon"]
SlamAndAwe.priorityTable.test["shw"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shockwave ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shockwave ID"])
  end
end

SlamAndAwe.priorityTable.name["ex"] = SlamAndAwe.constants["Execute"]
SlamAndAwe.priorityTable.icon["ex"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
  end
end

SlamAndAwe.priorityTable.name["ex_apl1"] = "Execute (if=buff.enrage | buff.battle_cry | buff.stone_heart)"
SlamAndAwe.priorityTable.icon["ex_apl1"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local EN = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local SH = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Stone Heart ID"])
    if (EN or BC or SH) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_apl2"] = "Execute (if=rage>50|buff.battle_cry|buff.stone_heart)"
SlamAndAwe.priorityTable.icon["ex_apl2"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_apl2"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local SH = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Stone Heart ID"])
    local rage = UnitPower("player", "rage")
    if (BC or SH or rage >= 50) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_apl3"] = "Execute (if=debuff.cs or rage >= 100)"
SlamAndAwe.priorityTable.icon["ex_apl3"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_apl3"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local rage = UnitPower("player", "rage")
    if (CS or rage >= 100) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_apl4"] = "Execute (if=debuff.cs & (buff.shattered_defenses | rage >= 100|tal.dc & buff.bc))"
SlamAndAwe.priorityTable.icon["ex_apl4"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_apl4"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local rage = UnitPower("player", "rage")
    if (CS and (SD or rage >= 100 or (talDeadlyCalm and BC))) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_sh"] = "Execute (if=buff.stone_heart)"
SlamAndAwe.priorityTable.icon["ex_sh"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_sh"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local SH = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Stone Heart ID"])
    if SH then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_csr"] = "Execute (if=debuff.cs | rage => 100)"
SlamAndAwe.priorityTable.icon["ex_csr"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_csr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local rage = UnitPower("player", "rage")
    if CS and rage >= 100 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["ex_wtf"] = "Execute (if=enrage & (!massacre & !inner_rage) | massacre & !enrage | enrage & (tal.massacre & !inner_rage))"
SlamAndAwe.priorityTable.icon["ex_wtf"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_wtf"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local usable, nomana = IsUsableSpell(SlamAndAwe.constants["Execute ID"]);
    local available = GetSpellInfo(GetSpellInfo(SlamAndAwe.constants["Execute ID"]))
    if usable and available then
      local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
      if Enraged then
        if not SlamAndAwe.db.char.talMassacre and not SlamAndAwe.db.char.talInnerRage then
          return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
        elseif not SlamAndAwe.db.char.talInnerRage and SlamAndAwe.db.char.talMassacre then
          return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
        end
      else
        if SlamAndAwe.db.char.talMassacre then
          return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
        end
      end
    end
  end
end

SlamAndAwe.priorityTable.name["ex_wtf2"] = "Execute (if=buff.enrage & !frenzy | tal.frenzy | tal.massacre)"
SlamAndAwe.priorityTable.icon["ex_wtf2"] = SlamAndAwe.constants["Execute Icon"]
SlamAndAwe.priorityTable.test["ex_wtf2"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Execute ID"]) then
    local usable, nomana = IsUsableSpell(SlamAndAwe.constants["Execute ID"]);
    local available = GetSpellInfo(GetSpellInfo(SlamAndAwe.constants["Execute ID"]))
    if usable and available then
      local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
      if SlamAndAwe.db.char.talFrenzy or SlamAndAwe.db.char.talMassacre then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
      elseif Enraged and not SlamAndAwe.db.char.talFrenzy then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Execute ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["het"] = SlamAndAwe.constants["Heroic Throw"]
SlamAndAwe.priorityTable.icon["het"] = SlamAndAwe.constants["Heroic Throw Icon"]
SlamAndAwe.priorityTable.test["het"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Heroic Throw ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Heroic Throw ID"])
  end
end

--Protection

SlamAndAwe.priorityTable.name["shs"] = SlamAndAwe.constants["Shield Slam"]
SlamAndAwe.priorityTable.icon["shs"] = SlamAndAwe.constants["Shield Slam Icon"]
SlamAndAwe.priorityTable.test["shs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shield Slam ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shield Slam ID"])
  end
end

SlamAndAwe.priorityTable.name["shs_fr"] = "Shield Slam (if=buff.focused_rage)"
SlamAndAwe.priorityTable.icon["shs_fr"] = SlamAndAwe.constants["Shield Slam Icon"]
SlamAndAwe.priorityTable.test["shs_fr"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shield Slam ID"]) then
    local FR, FRduration, FRstack = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
    if FR then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shield Slam ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["shs_fr3"] = "Shield Slam (if=buff.focused_rage=3)"
SlamAndAwe.priorityTable.icon["shs_fr3"] = SlamAndAwe.constants["Shield Slam Icon"]
SlamAndAwe.priorityTable.test["shs_fr3"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shield Slam ID"]) then
    local FR, FRduration, FRstack = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
    if FRstack == 3 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shield Slam ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["shbl"] = "Shield Block (if=!buff.sb)"
SlamAndAwe.priorityTable.icon["shbl"] = SlamAndAwe.constants["Shield Block Icon"]
SlamAndAwe.priorityTable.test["shbl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shield Block ID"]) and SlamAndAwe:IsOffGCD() then
    local SB, SBduration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shield Block ID"])
    if not SB then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shield Block ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["shbl_ip"] = "Shield Block (if=ip.duration>6 & sb.duration<1 & off.gcd)"
SlamAndAwe.priorityTable.icon["shbl_ip"] = SlamAndAwe.constants["Shield Block Icon"]
SlamAndAwe.priorityTable.test["shbl_ip"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Shield Block ID"]) and SlamAndAwe:IsOffGCD() then
    local IP, IPduration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Ignore Pain ID"])
    local SB, SBduration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shield Block ID"])
    if IPduration >= 6 and SBduration <= 1 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Shield Block ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["igpa"] = SlamAndAwe.constants["Ignore Pain"]
SlamAndAwe.priorityTable.icon["igpa"] = SlamAndAwe.constants["Ignore Pain Icon"]
SlamAndAwe.priorityTable.test["igpa"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Ignore Pain ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Ignore Pain ID"])
  end
end

SlamAndAwe.priorityTable.name["igpa_ven"] = "Ignore Pain (if=buff.vengeance)"
SlamAndAwe.priorityTable.icon["igpa_ven"] = SlamAndAwe.constants["Ignore Pain Icon"]
SlamAndAwe.priorityTable.test["igpa_ven"] =
function ()
  if SlamAndAwe.db.char.talVengeance then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Ignore Pain ID"]) then
      local venge = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Vengeance: Ignore Pain ID"])
      if venge then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Ignore Pain ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["igpa_du"] = "Ignore Pain (if=duration <= 3)"
SlamAndAwe.priorityTable.icon["igpa_du"] = SlamAndAwe.constants["Ignore Pain Icon"]
SlamAndAwe.priorityTable.test["igpa_du"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Ignore Pain ID"]) then
    local IP, IPduration = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Ignore Pain ID"])
    if (not IP or IPduration <= 3) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Ignore Pain ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["igpa_ra"] = "Ignore Pain (if=rage >= 60)"
SlamAndAwe.priorityTable.icon["igpa_ra"] = SlamAndAwe.constants["Ignore Pain Icon"]
SlamAndAwe.priorityTable.test["igpa_ra"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Ignore Pain ID"]) then
    local rage = UnitPower("player", "rage")
    if rage >= 60 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Ignore Pain ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["deva"] = SlamAndAwe.constants["Devastate"]
SlamAndAwe.priorityTable.icon["deva"] = SlamAndAwe.constants["Devastate Icon"]
SlamAndAwe.priorityTable.test["deva"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Devastate ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Devastate ID"])
  end
end

SlamAndAwe.priorityTable.name["reve"] = SlamAndAwe.constants["Revenge"]
SlamAndAwe.priorityTable.icon["reve"] = SlamAndAwe.constants["Revenge Icon"]
SlamAndAwe.priorityTable.test["reve"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Revenge ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Revenge ID"])
  end
end

SlamAndAwe.priorityTable.name["thcl"] = SlamAndAwe.constants["Thunder Clap"]
SlamAndAwe.priorityTable.icon["thcl"] = SlamAndAwe.constants["Thunder Clap Icon"]
SlamAndAwe.priorityTable.test["thcl"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Thunder Clap ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Thunder Clap ID"])
  end
end

SlamAndAwe.priorityTable.name["thcl_aoe"] = "Thunder Clap (if=adds >= 3)"
SlamAndAwe.priorityTable.icon["thcl_aoe"] = SlamAndAwe.constants["Thunder Clap Icon"]
SlamAndAwe.priorityTable.test["thcl_aoe"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Thunder Clap ID"]) then
    local Adds8 = SlamAndAwe:CountNameplates(8)
    local Adds15 = SlamAndAwe:CountNameplates(15)
    if Adds8 >= 3 or (SlamAndAwe.db.char.talCracklingThunder and Adds15 >= 3) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Thunder Clap ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hams"] = "Hamstring (if=!debuff.hamstring or duration <= 2)"
SlamAndAwe.priorityTable.icon["hams"] = SlamAndAwe.constants["Hamstring Icon"]
SlamAndAwe.priorityTable.test["hams"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Hamstring ID"]) then
    local hamsPresent, hamsDuration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Hamstring ID"])
    if not hamsPresent or hamsDuration <= 2 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Hamstring ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["hams_apl1"] = "Hamstring (if=tal.deadly_calm & buff.battle_cry & offgcd)"
SlamAndAwe.priorityTable.icon["hams_apl1"] = SlamAndAwe.constants["Hamstring Icon"]
SlamAndAwe.priorityTable.test["hams_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Hamstring ID"]) and SlamAndAwe:IsOffGCD() then
    local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    if talDeadlyCalm and BC then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Hamstring ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi"] = SlamAndAwe.constants["Whirlwind"]
SlamAndAwe.priorityTable.icon["whwi"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
  end
end

SlamAndAwe.priorityTable.name["whwi_mc"] = "Whirlwind (if=!buff.meat_cleaver)"
SlamAndAwe.priorityTable.icon["whwi_mc"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_mc"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    local MC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Meat Cleaver ID"])
    if not MC then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_60"] = "Whirlwind (if=rage >= 60)"
SlamAndAwe.priorityTable.icon["whwi_60"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_60"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    local rage = UnitPower("player", "rage")
    if rage >= 60 then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_ir"] = "Whirlwind (if=!inner_rage & buff.wrecking_ball)"
SlamAndAwe.priorityTable.icon["whwi_ir"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_ir"] =
function ()
  if not SlamAndAwe.db.char.talInnerRage then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
      local rbPresent = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Wrecking Ball ID"])
      return rbPresent and SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_wb"] = "Whirlwind (if=buff.wrecking_ball)"
SlamAndAwe.priorityTable.icon["whwi_wb"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_wb"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    local rbPresent = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Wrecking Ball ID"])
    return rbPresent and SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
  end
end

SlamAndAwe.priorityTable.name["whwi_wben"] = "Whirlwind (if=buff.wrecking_ball & buff.enrage)"
SlamAndAwe.priorityTable.icon["whwi_wben"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_wben"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    local Enraged = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"])
    if Enraged then
      local rbPresent = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Wrecking Ball ID"])
      return rbPresent and SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_apl1"] = "Whirlwind (if=tal.fob & debuff.cs & !tal.fr | tal.dc & (buff.bc | buff.cleave))"
SlamAndAwe.priorityTable.icon["whwi_apl1"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local CL, CLduration, CLcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Cleave ID"])
    local BC, BCduration, BCcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local rage = UnitPower("player", "rage")
    if (SlamAndAwe.db.char.talFervorOfBattle and CS and not SlamAndAwe.db.char.talFocusedRage) or (talDeadlyCalm and (BC or CL)) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_apl2"] = "Whirlwind (if=tal.fob & (!tal.fr | rage>100 | fr.stack=3))"
SlamAndAwe.priorityTable.icon["whwi_apl2"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_apl2"] =
function ()
  if talFervorOfBattle then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Whirlwind ID"]) then
      local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
      local rage = UnitPower("player", "rage")
      if rage >= 100 or FRcount == 3 or not talFocusedRage then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Whirlwind ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["whwi_apl3"] = "Whirlwind (if=tal.fob & (debuff.cs | rage.deficit < 50) & !tall.fr | tal.dc & (buff.bc | buff.cleave))"
SlamAndAwe.priorityTable.icon["whwi_apl3"] = SlamAndAwe.constants["Whirlwind Icon"]
SlamAndAwe.priorityTable.test["whwi_apl3"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Slam ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    local CL, CLduration, CLcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Cleave ID"])
    local BC, BCduration, BCcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry ID"])
    local rage = UnitPower("player", "rage")
    local maxrage = UnitPowerMax("player" , "rage")
    if (SlamAndAwe.db.char.talFervorOfBattle and (CS or rage > maxrage - 50) and not SlamAndAwe.db.char.talFocusedRage) or (SlamAndAwe.db.char.talDeadlyCalm and (BC or CL)) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Slam ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["desh"] = SlamAndAwe.constants["Demoralizing Shout"]
SlamAndAwe.priorityTable.icon["desh"] = SlamAndAwe.constants["Demoralizing Shout Icon"]
SlamAndAwe.priorityTable.test["desh"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Demoralizing Shout ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Demoralizing Shout ID"])
  end
end

SlamAndAwe.priorityTable.name["most"] = SlamAndAwe.constants["Mortal Strike"]
SlamAndAwe.priorityTable.icon["most"] = SlamAndAwe.constants["Mortal Strike Icon"]
SlamAndAwe.priorityTable.test["most"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Mortal Strike ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
  end
end

SlamAndAwe.priorityTable.name["most_iftks20"] = "Mortal Strike (if=subtwenty & tal.InForTheKill & !buff.shattered_defenses)"
SlamAndAwe.priorityTable.icon["most_iftks20"] = SlamAndAwe.constants["Mortal Strike Icon"]
SlamAndAwe.priorityTable.test["most_iftks20"] =
function ()
  if SlamAndAwe.db.char.talInForTheKill then
    if SlamAndAwe:SpellReady(SlamAndAwe.constants["Mortal Strike ID"]) then
      local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
      if SlamAndAwe:TargetSubTwenty() and not SD then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["most_s20"] = "Mortal Strike (if=subtwenty)"
SlamAndAwe.priorityTable.icon["most_s20"] = SlamAndAwe.constants["Mortal Strike Icon"]
SlamAndAwe.priorityTable.test["most_s20"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Mortal Strike ID"]) and SlamAndAwe:TargetSubTwenty() then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
  end
end

SlamAndAwe.priorityTable.name["most_apl1"] = "Mortal Strike (if=!subtwenty|subtwenty & buff.shattered_defenses & buff.fr = 3)"
SlamAndAwe.priorityTable.icon["most_apl1"] = SlamAndAwe.constants["Mortal Strike Icon"]
SlamAndAwe.priorityTable.test["most_apl1"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Mortal Strike ID"]) and SlamAndAwe:TargetSubTwenty() then
    local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
    local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
    if (SD and FRcount == 3) then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
    end
  else
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
  end
end

SlamAndAwe.priorityTable.name["most_fr3"] = "Mortal Strike (if=buff.focused_rage.stack = 3 & debuff.colossus_smash)"
SlamAndAwe.priorityTable.icon["most_fr3"] = SlamAndAwe.constants["Mortal Strike Icon"]
SlamAndAwe.priorityTable.test["most_fr3"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Mortal Strike ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if CS then
      local FR, FRduration, FRcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Focused Rage ID"])
      if FRcount == 3 then
        return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Mortal Strike ID"])
      end
    end
  end
end

SlamAndAwe.priorityTable.name["ovpo"] = SlamAndAwe.constants["Overpower"]
SlamAndAwe.priorityTable.icon["ovpo"] = SlamAndAwe.constants["Overpower Icon"]
SlamAndAwe.priorityTable.test["ovpo"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Overpower ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Overpower ID"])
  end
end

SlamAndAwe.priorityTable.name["war"] = SlamAndAwe.constants["Warbreaker"]
SlamAndAwe.priorityTable.icon["war"] = SlamAndAwe.constants["Warbreaker Icon"]
SlamAndAwe.priorityTable.test["war"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Warbreaker ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Warbreaker ID"])
  end
end

SlamAndAwe.priorityTable.name["war_ncs"] = "Warbreaker (if=!debuff.colossus_smash)"
SlamAndAwe.priorityTable.icon["war_ncs"] = SlamAndAwe.constants["Warbreaker Icon"]
SlamAndAwe.priorityTable.test["war_ncs"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Warbreaker ID"]) then
    local CS = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash Debuff ID"])
    if not CS then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Warbreaker ID"])
    end
  end
end

SlamAndAwe.priorityTable.name["war_nsd"] = "Warbreaker (if=!buff.shattered_defenses)"
SlamAndAwe.priorityTable.icon["war_nsd"] = SlamAndAwe.constants["Warbreaker Icon"]
SlamAndAwe.priorityTable.test["war_nsd"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Warbreaker ID"]) then
    local SD = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Shattered Defenses ID"])
    if not SD then
      return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Warbreaker ID"])
    end
  end
end

--/run print(GetSpellCooldown("Bladestorm")) print(SlamAndAwe:SpellCooldown("Bladestorm")) print(IsUsableSpell("Bladestorm"))
SlamAndAwe.priorityTable.name["blst"] = SlamAndAwe.constants["Bladestorm"]
SlamAndAwe.priorityTable.icon["blst"] = SlamAndAwe.constants["Bladestorm Icon"]
SlamAndAwe.priorityTable.test["blst"] =
function ()
  if SlamAndAwe:SpellReady(SlamAndAwe.constants["Bladestorm ID"]) then
    return SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Bladestorm ID"])
  end
end

-------------------------------
-- Priority Frame functions
-------------------------------

function SlamAndAwe:SetPriorityBackdrop(icon)

  if SlamAndAwe.db.char.priority.enableDisableBorder then

    self.PriorityFrame:SetBackdrop({ bgFile = icon
      })
  else
    --self.PriorityFrame.Icon:Set(icon)

    self.PriorityFrame:SetBackdrop({ bgFile = icon,
        edgeFile = media:Fetch("border", SlamAndAwe.db.char.priority.barborder),
        tile = false, tileSize = 0, edgeSize = 12,
        insets = { left = 2, right = 2, top = 2, bottom = 2 }
      })

    if SlamAndAwe.db.char.currentspec == "Fury" then
      if SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Enrage ID"]) then
        self.PriorityFrame:SetBackdropBorderColor(1,0,0,1)
      else
        self.PriorityFrame:SetBackdropBorderColor(1,1,1,1)
      end
    end

  end
end

function SlamAndAwe:CreatePriorityFrame()
  self.updatePriority = false
  self.PriorityFrame:SetScale(SlamAndAwe.db.char.priority.scale)
  self.PriorityFrame:SetFrameStrata(SlamAndAwe.db.char.framestratalevel)
  self.PriorityFrame:SetWidth(SlamAndAwe.db.char.priority.fWidth)
  self.PriorityFrame:SetHeight(SlamAndAwe.db.char.priority.fHeight)
  self.PriorityFrame.Icon = _G["SAA_PriorityFrameIcon"]
  self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
  self.PriorityFrame:SetBackdropColor(0, 0, 0, SlamAndAwe.db.char.priority.alpha);
  self.PriorityFrame:SetMovable(true);
  self.PriorityFrame:RegisterForDrag("LeftButton");
  self.PriorityFrame:EnableMouse(false)
  self.PriorityFrame:SetPoint(SlamAndAwe.db.char.priority.point, SlamAndAwe.db.char.priority.relativeTo, SlamAndAwe.db.char.priority.relativePoint, SlamAndAwe.db.char.priority.xOffset, SlamAndAwe.db.char.priority.yOffset)
  self.PriorityFrame:SetScript("OnDragStart",
    function()
      self.PriorityFrame:StartMoving();
    end );
  self.PriorityFrame:SetScript("OnDragStop",
    function()
      self.PriorityFrame:StopMovingOrSizing();
      self.PriorityFrame:SetScript("OnUpdate", nil);
      self:FinishedMoving(SlamAndAwe.db.char.priority, self.PriorityFrame);
    end );
  self:SetPriorityUpdateScript()
  if not self.PriorityFrame.topText then
    self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
  end
  self.PriorityFrame.topText:SetTextColor(1,1,1,1)
  self.PriorityFrame.topText:SetFont(media:Fetch("font", SlamAndAwe.db.char.barfont), SlamAndAwe.db.char.barfontsize)
  self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, SlamAndAwe.db.char.barfontsize + 2)
  self.PriorityFrame.topText:SetText(string.format(L["Next Priority (Set %s)"], SlamAndAwe.db.char.priority.groupnumber))
  self.PriorityFrame.cooldown:SetAllPoints(self.PriorityFrame)
  --self.PriorityFrame.cooldown:SetSwipeTexture('Interface/Cooldown/edge')
  -- self.PriorityFrame.cooldown:SetSwipeColor(0, 0, 0, 0.8)
  self:CreateInterruptPurgeFrames()

  if SlamAndAwe.db.char.priority.show and InCombatLockdown() then
    self.PriorityFrame:Show()
  else
    self.PriorityFrame:Hide()
  end
  if SlamAndAwe.db.char.priority.titleshow then
    self.PriorityFrame.topText:Show()
  else
    self.PriorityFrame.topText:Hide()
  end
  if LBF and LBFgroup then
    LBFgroup:AddButton(self.PriorityFrame)
  end
end

function SlamAndAwe:ResetPriority()
  self.PriorityFrame:ClearAllPoints()
  SlamAndAwe.db.char.priority.point = self.defaults.char.priority.point
  SlamAndAwe.db.char.priority.relativeTo = self.defaults.char.priority.relativeTo
  SlamAndAwe.db.char.priority.relativePoint = self.defaults.char.priority.relativePoint
  SlamAndAwe.db.char.priority.xOffset = self.defaults.char.priority.xOffset
  SlamAndAwe.db.char.priority.yOffset = self.defaults.char.priority.yOffset
  SlamAndAwe.db.char.priority.fWidth = self.defaults.char.priority.fWidth
  SlamAndAwe.db.char.priority.fHeight = self.defaults.char.priority.fHeight
  SlamAndAwe.db.char.priority.scale = self.defaults.char.priority.scale
  SlamAndAwe.db.char.priority.prOption = self.defaults.char.priority.prOption
  self.PriorityFrame:SetPoint(SlamAndAwe.db.char.priority.point, SlamAndAwe.db.char.priority.relativeTo, SlamAndAwe.db.char.priority.relativePoint, SlamAndAwe.db.char.priority.xOffset, SlamAndAwe.db.char.priority.yOffset)
  self:CreatePriorityFrame()
  self:Print(L["priority_reset"])
end

function SlamAndAwe:SetPriorityUpdateScript()
  self.PriorityFrame:SetScript("OnUpdate",
    function()
      if self.updatePriority then
        SlamAndAwe.db.char.priority.previous = SlamAndAwe.db.char.priority.next
        SlamAndAwe:SetNextPriority()
        -- if SlamAndAwe.db.char.priority.next ~= SlamAndAwe.db.char.priority.previous or SlamAndAwe:CheckShamanisticRage() then
        SlamAndAwe:SetPriorityIcon(SlamAndAwe.db.char.priority.next) -- always set icon to fix the times where it was getting stuck
        -- end
        if SlamAndAwe.db.char.priority.showcooldown then
          local startTime, duration, enabled = GetSpellCooldown(SlamAndAwe.constants["SpellCoolDown"])
          if startTime then
            -- SlamAndAwe:DebugPrint("Cooldown "..startTime.. " - "..duration)

            SlamAndAwe.PriorityFrame.cooldown:SetCooldown(startTime, duration)
          end
        end
      end

      if SlamAndAwe.db.char.priority.show and not SlamAndAwe.db.char.disabled then
        if SlamAndAwe.db.char.priority.showinterrupt and self.PriorityFrame.interrupt then
          local EnemySpell, _, _, EnemySpellIcon, _, _, _, _, EnemySpellInterruptable = UnitCastingInfo("target")
          EnemySpellInterruptable = not EnemySpellInterruptable --reverse since false actually means interruptable
          if not EnemySpell then
            -- OBU.EnemySpellIcon = "Interface/Tooltips/UI-Tooltip-Background"
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
          end

          --This section causes high memory allocation?
          self.PriorityFrame.interrupt.frame:SetBackdrop({ bgFile = EnemySpellIcon,
              edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
              tile = false, tileSize = 0, edgeSize = 12,
              insets = { left = 2, right = 2, top = 2, bottom = 2 }
            })
          --------

          self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 1, 1, 1);

          local usable, nomana = IsUsableSpell(SlamAndAwe.constants["Pummel ID"]);
          local available = GetSpellCooldown(GetSpellInfo(SlamAndAwe.constants["Pummel ID"]))
          if EnemySpellInterruptable then
            if (usable and available == 0) then
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 1, 0, 1);
              -- self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 4)
              -- self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1,0,0,1)
            else
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1);
              --self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
              --self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 12)
              --self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1); -- set border color to Red if Purge not available
            end
          end
          if not EnemySpell then
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
            self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 0, 0, 0);
          end
        end
      end

    end );
end

function SlamAndAwe:SetPriorityIcon(priority)
  --SlamAndAwe:DebugPrint("setting icon to "..priority)
  if InCombatLockdown() or priority == "none" then
    local icon = self.priorityTable.icon[priority]
    -- if self:CheckWindShear() then
    -- SlamAndAwe:SetSubFrameBackdrop(SlamAndAwe.PriorityFrame.purge.frame, SlamAndAwe.constants["Focused Rage Icon"], 4)
    -- else
    -- self:SetSubFrameBackdrop(self.PriorityFrame.purge.frame, "Interface/Tooltips/UI-Tooltip-Background", 4)
    -- self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
    -- end
    self:SetPriorityBackdrop(icon)
  end
end

function SlamAndAwe:CheckWindShear()
  if SlamAndAwe.db.char.hsshow then
    -- local UL = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Ultimatum"])
    -- if UL then
    -- return true
    -- else
    -- local avail = SlamAndAwe:SpellAvailable(SlamAndAwe.constants["Heroic Strike"])
    -- local usable, nomana = IsUsableSpell(SlamAndAwe.constants["Heroic Strike"])
    -- if usable and avail then
    -- local TargetSubTwenty = SlamAndAwe:TargetSubTwenty()
    -- local CS, CSduration = SlamAndAwe:GetDebuffInfo(SlamAndAwe.constants["Colossus Smash"])
    -- local TFB, TFBduration, TFBcount = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Taste for Blood"])
    -- local BC = SlamAndAwe:GetBuffInfo(SlamAndAwe.constants["Battle Cry"])
    -- local rage = UnitPower("player", "rage")

    -- if rage >= SlamAndAwe.db.char.rageThreshold then
    -- return true
    -- elseif SlamAndAwe.db.char.currentspec == "Fury" then
    --Fury
    -- if not TargetSubTwenty then
    -- -- Target below 20% health
    -- if CS then
    -- if rage >= 40 then
    -- return true
    -- end
    -- elseif (DC and CS) then
    -- if rage >= 30 then
    -- return true
    -- end
    -- end
    -- end
    --End Fury
    -- elseif SlamAndAwe.db.char.currentspec == "Arms" then
    --Arms
    -- if not TargetSubTwenty then
    -- if ((TFB and TFBduration <=2) or (TFBcount == 5) or (TFB and CSduration <= 2) or BC) and CS then
    -- return true
    -- end
    -- end
    -- end
    -- end
    -- end
  end
end

function SlamAndAwe:SelectPrioritySet(groupNumber)
  SlamAndAwe.db.char.priority.prOption = SlamAndAwe.db.char.priority.prOptions[groupNumber]
  SlamAndAwe.db.char.priority.groupnumber = groupNumber
  if self.PriorityFrame and self.PriorityFrame.topText then
    self.PriorityFrame.topText:SetText(string.format(L["Next Priority (Set %s)"], SlamAndAwe.db.char.priority.groupnumber))
  end
  --self:Print(string.format(L["Priority Set %s selected"], groupNumber)) --removed to prevent spamming during setnextpriority
end

function SlamAndAwe:SetNextPriority()
  -- force update of stacks count to ensure priority shown correctly
  -- SlamAndAwe.db.char.msstacks = SlamAndAwe:GetMaelstromInfo()
  -- if SlamAndAwe.db.char.priority.combopoints then
  -- self:ShowComboPoints()
  -- end
  local targets = SlamAndAwe:CountNameplates(8)

  if SlamAndAwe.db.char.currentspec == "Arms" then
    if targets >= SlamAndAwe.db.char.priority.armsaoethreshold and SlamAndAwe.db.char.priority.groupnumber ~= 5 then
      SlamAndAwe:SelectPrioritySet(5)
    elseif targets < SlamAndAwe.db.char.priority.armsaoethreshold and SlamAndAwe.db.char.priority.groupnumber ~= 3 then
      SlamAndAwe:SelectPrioritySet(3)
    end
  elseif SlamAndAwe.db.char.currentspec == "Fury" then
    if targets >= SlamAndAwe.db.char.priority.furyaoethreshold and SlamAndAwe.db.char.priority.groupnumber ~= 4 then
      SlamAndAwe:SelectPrioritySet(4)
    elseif targets < SlamAndAwe.db.char.priority.furyaoethreshold and SlamAndAwe.db.char.priority.groupnumber ~= 1 then
      SlamAndAwe:SelectPrioritySet(1)
    end
  end

  SlamAndAwe.db.char.priority.next = "none"
  for index = 1, 30 do
    if SlamAndAwe.db.char.priority.prOption[index] and self.priorityTable.test[SlamAndAwe.db.char.priority.prOption[index]] then -- verify that the option actually exists
      if self.priorityTable.test[SlamAndAwe.db.char.priority.prOption[index]]() then
        SlamAndAwe.db.char.priority.next = SlamAndAwe.db.char.priority.prOption[index]
        return -- we need to break out of the routine as we have found the top priority
      end
    end
  end
end

function SlamAndAwe:TargetSubTwenty()
  local dechp
  local perchp
  local calchp
  local currenthp = UnitHealth("target")
  local totalhp = UnitHealthMax("target")
  if currenthp == 0 then
    dechp = 0
    perchp = 0
  elseif currenthp == totalhp then
    perchp = 100
  else
    calchp = currenthp / totalhp
    dechp = calchp * 100
    perchp = math.ceil(dechp)
  end

  if perchp <= 20 then
    return true
  else
    return false
  end
end

function SlamAndAwe:SelfSubTwenty()
  local selfcurrenthp = UnitHealth("player")
  local selftotalhp = UnitHealthMax("player")
  local selfdechp
  local selfperchp
  local selfcalchp
  if selfcurrenthp == 0 then
    selfdechp = 0
    selfperchp = 0
  elseif currenthp == totalhp then
    selfperchp = 100
  else
    local selfcalchp = selfcurrenthp / selftotalhp
    local selfdechp = selfcalchp * 100
    selfperchp = math.ceil(dechp)
  end

  if selfperchp <= 20 then
    return true
  else
    return false
  end
end

function SlamAndAwe:CreateInterruptPurgeFrames()

  local width = SlamAndAwe.db.char.priority.fWidth / 1.6
  local height = SlamAndAwe.db.char.priority.fHeight / 1.6

  if not self.PriorityFrame.interrupt then
    self.PriorityFrame.interrupt = {}
    self.PriorityFrame.interrupt.frame = CreateFrame("Frame", "SAA_PriorityInterruptFrame", self.PriorityFrame)
  end
  local interruptFrame = self.PriorityFrame.interrupt.frame
  interruptFrame:SetScale(SlamAndAwe.db.char.priority.scale)
  interruptFrame:SetFrameStrata("BACKGROUND")
  interruptFrame:SetWidth(width)
  interruptFrame:SetHeight(height)

  --SlamAndAwe:SetTexture("Interface/Tooltips/UI-Tooltip-Background", false)

  self:SetSubFrameBackdrop(interruptFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  interruptFrame:SetBackdropColor(0, 0, 0, 0);
  interruptFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if SlamAndAwe.db.char.priority.showinterrupt then
    interruptFrame:Show()
  else
    interruptFrame:Hide()
  end
  SlamAndAwe:SetSubFrameBackdrop(SlamAndAwe.PriorityFrame.interrupt.frame, SlamAndAwe.constants["Focused Rage Icon"], 4)
  if not self.PriorityFrame.purge then
    self.PriorityFrame.purge = {}
    self.PriorityFrame.purge.frame = CreateFrame("Frame", "SAA_PrioritypurgeFrame", self.PriorityFrame)
  end

  local purgeFrame = self.PriorityFrame.purge.frame
  purgeFrame:SetScale(SlamAndAwe.db.char.priority.scale)
  purgeFrame:SetFrameStrata("BACKGROUND")
  purgeFrame:SetWidth(width)
  purgeFrame:SetHeight(height)
  self:SetSubFrameBackdrop(purgeFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  purgeFrame:SetBackdropColor(0, 0, 0, 0);
  purgeFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if SlamAndAwe.db.char.priority.showpurge then
    purgeFrame:Show()
  else
    purgeFrame:Hide()
  end
end

--[[
function SlamAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      edgeSize = edgeSize
    })
end
]]
function SlamAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      tile = false, tileSize = 0, edgeSize = edgeSize

    })
end
